@@ -789,7 +789,7 @@ getColumnsInfo(ConnectionClass *conn, TABLE_INFO *wti, OID greloid, StatementCla
 
 	if (NULL == conn)
 		conn = SC_get_conn(stmt);
-	result = PGAPI_AllocStmt(conn, &hcol_stmt);
+	result = PGAPI_AllocStmt(conn, &hcol_stmt, 0);
 	if (!SQL_SUCCEEDED(result))
 	{
 		if (stmt)
@@ -1091,7 +1091,7 @@ inolog("%s:fields=%d ntab=%d\n", func, nfields, stmt->ntab);
 		char		keycolnam[MAX_INFO_STRING];
 		SQLLEN		keycollen;
 
-		ret = PGAPI_AllocStmt(conn, &pstmt);
+		ret = PGAPI_AllocStmt(conn, &pstmt, 0);
 		if (!SQL_SUCCEEDED(ret))
 			return ret;
 		oneti = ti[0];
@@ -1208,7 +1208,7 @@ static char
 parse_the_statement(StatementClass *stmt, BOOL check_hasoids, BOOL sqlsvr_check)
 {
 	CSTR		func = "parse_the_statement";
-	char		token[TOKEN_SIZE], stoken[TOKEN_SIZE], btoken[TOKEN_SIZE];
+	char		delimdsp[2], token[TOKEN_SIZE], stoken[TOKEN_SIZE], btoken[TOKEN_SIZE];
 	char		delim,
 				quote,
 				dquote,
@@ -1280,7 +1280,14 @@ parse_the_statement(StatementClass *stmt, BOOL check_hasoids, BOOL sqlsvr_check)
 	{
 		unquoted = !(quote || dquote);
 
-		mylog("unquoted=%d, quote=%d, dquote=%d, numeric=%d, delim='%c', token='%s', ptr='%s'\n", unquoted, quote, dquote, numeric, delim, token, ptr);
+		if (delim)
+		{
+			delimdsp[0] = delim;
+			delimdsp[1] = '\0';
+		}
+		else
+			delimdsp[0] = '\0';
+		mylog("unquoted=%d, quote=%d, dquote=%d, numeric=%d, delim='%s', token='%s', ptr='%s'\n", unquoted, quote, dquote, numeric, delimdsp, token, ptr);
 
 		old_blevel = blevel;
 		if (unquoted && blevel == 0)
