@@ -376,41 +376,36 @@ getNumericDecimalDigitsX(const ConnectionClass *conn, OID type, int atttypmod, i
 		return (atttypmod & 0xffff);
 	if (adtsize_or_longest <= 0)
 		return default_decimal_digits;
-	if (UNKNOWNS_AS_CATALOG != handle_unknown_size_as)
-	{
-		/* if (adtsize_or_longest < 5)
-			adtsize_or_longest = 5; */
-		adtsize_or_longest >>= 16; /* extract the scale part */
-	}
+	adtsize_or_longest >>= 16; /* extract the scale part */
 	return adtsize_or_longest;
 }
 
 static Int4	/* PostgreSQL restritiction */
 getNumericColumnSizeX(const ConnectionClass *conn, OID type, int atttypmod, int adtsize_or_longest, int handle_unknown_size_as)
 {
-	Int4	default_column_size = 0; /* 28; */
+	Int4	default_column_size = 28;
 
 	mylog("%s: type=%d, typmod=%d\n", __FUNCTION__, type, atttypmod);
 
-	if (atttypmod < 0 && adtsize_or_longest < 0)
-		return default_column_size;
-
 	if (atttypmod > -1)
 		return (atttypmod >> 16) & 0xffff;
-	if (adtsize_or_longest <= 0)
-		return default_column_size;
 	switch (handle_unknown_size_as)
 	{
 		case UNKNOWNS_AS_DONTKNOW:
-			return 0;
-		case UNKNOWNS_AS_MAX:
-			return default_column_size;
+			return SQL_NO_TOTAL;
 	}
+	if (adtsize_or_longest <= 0)
+		return default_column_size;
 	adtsize_or_longest %= (1 << 16); /* extract the precision part */
-	if (UNKNOWNS_AS_CATALOG != handle_unknown_size_as)
+	switch (handle_unknown_size_as)
 	{
-		if (adtsize_or_longest < 10)
-			adtsize_or_longest = 10;
+		case UNKNOWNS_AS_MAX:
+			return adtsize_or_longest > default_column_size ? adtsize_or_longest : default_column_size;
+		case UNKNOWNS_AS_CATALOG:
+			break;
+		default: 
+			if (adtsize_or_longest < 10)
+				adtsize_or_longest = 10;
 	}
 	return adtsize_or_longest;
 }
@@ -1443,12 +1438,11 @@ getAtttypmodEtc(const StatementClass *stmt, int col, int *adtsize_or_longestlen)
 
 						for (i = 0; i < res->num_cached_rows; i++)
 						{
-							if (tval = QR_get_value_backend_text(res, col, i), NULL != tval)
+							if (tval = QR_get_value_backend_text(res, i, col), NULL != tval)
 							{
 								if (sptr = strchr(tval, '.'), NULL != sptr)
 									if (sval = strlen(tval) - (sptr + 1 - tval), sval > maxscale)
 										maxscale = sval;
-									
 							}
 						}
 						*adtsize_or_longestlen += (maxscale << 16);
